@@ -50,6 +50,13 @@ def get_configured_subject_class(subject_class_name=None):
 
     config = mi_utils.MIResearch_config
     subject_class_name = subject_class_name or getattr(config, 'subject_class_name', None)
+    if (subject_class_name is None) or (subject_class_name == ""):
+        subject_obj = getattr(mi_utils.MIResearch_config, 'class_obj', None)
+        if subject_obj is not None:
+            if isinstance(subject_obj, str):
+                subject_class_name = subject_obj
+            else:
+                subject_class_name = subject_obj.__module__ + "." + subject_obj.__name__
 
     if not subject_class_name:
         _CACHED_SUBJECT_CLASS = _SENTINEL_CLASS_NOT_CONFIGURED_OR_FAILED
@@ -156,11 +163,13 @@ class AbstractSubject(object):
         self.dicomMetaTagListStudy = mi_utils.DEFAULT_DICOM_META_TAG_LIST_STUDY
         self.dicomMetaTagListSeries = mi_utils.DEFAULT_DICOM_META_TAG_LIST_SERIES
         self.QUIET = False
-        self.DEBUG = False
+        self.DEBUG = mi_utils.MIResearch_config
         #
         #
         self._logger = None
         self._loggerFH = None
+        # 
+        self.meta_cache = {}
 
 
     ### ----------------------------------------------------------------------------------------------------------------
@@ -240,7 +249,10 @@ class AbstractSubject(object):
         if self._logger is None:
             rr = os.path.split(self.dataRoot)[1]
             self._logger = logging.getLogger(f"{rr}/{self.subjID}")
-            self._logger.setLevel(logging.INFO)
+            if self.DEBUG:
+                self._logger.setLevel(logging.DEBUG)    
+            else:
+                self._logger.setLevel(logging.INFO)
             self._logger.propagate = False  # Set propagate to False by default
             
             # Remove any existing handlers
@@ -330,6 +342,7 @@ class AbstractSubject(object):
             for handler in self.logger.handlers[:]:
                 self.logger.removeHandler(handler)
                 handler.close()
+            self._logger = None
         except IOError:
             pass # If does not exist. 
 
@@ -631,8 +644,16 @@ class AbstractSubject(object):
         return self.getMetaTagValue(tagName, ifNotFound, metaSuffix)
 
 
+    def _cacheMeta(self, suffix):
+        ff = self.getMetaTagsFile(suffix)
+        dd = {}
+        if os.path.isfile(ff):
+            dd = fIO.parseJsonToDictionary(ff)
+        self.meta_cache[suffix] = dd
+
+
     def getMetaDict(self, suffix=""):
-        """Get meta json file as dictionary
+        """Get meta json file as dictionary. Will check if cached. 
 
         Args:
             suffix (str, optional): Suffix of json file. Defaults to "".
@@ -640,11 +661,9 @@ class AbstractSubject(object):
         Returns:
             dict: Meta json file 
         """
-        ff = self.getMetaTagsFile(suffix)
-        dd = {}
-        if os.path.isfile(ff):
-            dd = fIO.parseJsonToDictionary(ff)
-        return dd
+        if suffix not in self.meta_cache.keys():
+            self._cacheMeta(suffix=suffix)
+        return self.meta_cache[suffix]
 
 
     def getMetaTagValue(self, tag, NOT_FOUND=None, metaSuffix=""):
@@ -676,12 +695,17 @@ class AbstractSubject(object):
         Args:
             metaDict (dict): dictionary with key value pairs to update
             metasuffix (str, optional): Suffix of json file. Defaults to "".
+        
+        Returns:
+            str: Full path of file updated
         """
         dd = self.getMetaDict(metasuffix)
         dd.update(metaDict)
-        fIO.writeDictionaryToJSON(self.getMetaTagsFile(metasuffix), dd)
-        # spydcm.dcmTools.writeDictionaryToJSON(self.getMetaTagsFile(metasuffix), dd)
-        self.logger.info('Updated meta-file')
+        metaFile = self.getMetaTagsFile(metasuffix)
+        fIO.writeDictionaryToJSON(metaFile, dd)
+        self.logger.info(f'Updated meta-file')
+        self._cacheMeta(metasuffix)
+        return metaFile
 
 
     def buildDicomMeta(self):
@@ -1155,6 +1179,14 @@ class SubjectList(list):
         return cls(listOfSubjects)
 
 
+    @classmethod
+    def setBySNList(cls, snList, SubjClass=None):
+        if SubjClass is None:
+            SubjClass = get_configured_subject_class()
+        subjList = [SubjClass(i, mi_utils.MIResearch_config.data_root_dir, mi_utils.MIResearch_config.subject_prefix) for i in snList]
+        return cls(subjList)
+
+
     @property
     def subjIDs(self):
         return [i.subjID for i in self]
